@@ -441,12 +441,7 @@ static int at24_probe(struct device_d *dev)
 			dev_err(&client->dev, "no index found to name device\n");
 			goto err_device_name;
 		}
-		devname = basprintf("eeprom%d", err);
-		if (!devname) {
-			err = -ENOMEM;
-			dev_err(&client->dev, "failed to allocate space for device name\n");
-			goto err_device_name;
-		}
+		devname = xasprintf("eeprom%d", err);
 	}
 
 	at24->cdev.name = devname;
@@ -505,6 +500,7 @@ static int at24_probe(struct device_d *dev)
 		goto err_devfs_create;
 
 	of_parse_partitions(&at24->cdev, dev->device_node);
+	of_partitions_register_fixup(&at24->cdev);
 
 	return 0;
 
